#pragma rtGlobals=1		// Use modern global access method.
#pragma version=0.2.2	//November 12, 2007
#pragma IgorVersion=6
#include <WaveSelectorWidget>

//This procedure file provides functions to create a control panel-driven plot of the time-frequency content of a signal.
//The features are described in a separate help file.

//The procedure file relies on the AR(1), Interpolations, Spectral Estimates, and Time-Freq Decompositions procedure files.

//FOR PROGRAMMERS WANTING TO EXTEND THE FUNCTIONALITY:
//To add a module in a separate tab, see the TFAddTab_prototype function.
//To add interpolation, spectral estimate, or time-frequency decomposition methods, see the separate procedure files.

//written by Ben Cramer (bscramer@uoregon.edu)

static constant kMaxNameLength=33		//Igor names are restricted to 31 characters; add 2 for enclosing single-quotes in liberal names
static constant kTFVersion=022			//current Time-Frequency procedure version

strconstant KPopupvalsVertAxis="Linear frequency;Linear period;Log(10);Log(2)"
strconstant KPopupvalsAmpAxis="Linear;Log(10);Log(2)"
strconstant KPopupvalsNorms="Magnitude;AR normalized"
strconstant KPopupvalsSwapXY="vertical spectrum plot;vertical data plot"

Menu "Analysis"
	"Time-Frequency Investigation",/Q
end

Structure TF_GlobalRefs							//structure containing references to all globals, so they don't have to be redefined constantly
	
//waves
//data
	wave dw,xw,dw_int								//wave references for user selected data and x-value waves and for interpolated data
//decomposition
	wave TFmag,TFfrequency,TFperiod,TFscale,TFcoi,TFBGnorm,TFGlobalSpec
//spectrum
	wave AR1Mag,AR1Frequency,AR1Period				//wave references for expected AR1 red noise spectrum
	wave PgramMag,PgramPh,PgramFrequency,PgramPeriod	//wave references for periodogram
//appearance
	wave CTabMod										//wave reference for color table to display time-frequency decomposition
	wave/t vertP,vertF
	wave vertPy,vertFy								//manual tick waves
//for display
	wave PlotTFdecomp,PlotXTFdecomp
	wave PlotXAR1,PlotXPgram

//window names
	SVAR Win											//name of window
	string ctrlWin,dataWin							//tabbed control window, and data selection control window
	SVAR dwPath,xwPath
//preprocessing tab
//	SVAR DetrendType									//current setting of detrend choice
//	NVAR RemoveOutliers,RemoveFliers
//	NVAR RemoveSD,RemoveRG						//standard deviation and data window to define fliers/outliers
//interpolation tab
	SVAR InterpType									//current setting of TF interpolation popup selector
	NVAR LiveInterp									//auto-interpolate setting
	NVAR InterpRes,Xmin,Xmax						//interpolation parameters
	NVAR InterpSV0,InterpSV1,InterpSV2			//current setting of interpolation setvariable parameters
	SVAR InterpPM0,InterpPM1,InterpPM2		//current setting of interpolation popupmenu parameters
//decomposition tab
	SVAR DecompType									//current setting of TF decomposition popup selector
	NVAR PeriodMin,PeriodMax						//decomposition parameters
	NVAR freqRes
	NVAR TFDecompSV0,TFDecompSV1,TFDecompSV2	//current setting of decomposition setvariable parameters
	SVAR TFDecompPM0,TFDecompPM1,TFDecompPM2	//current setting of decomposition popupmenu parameters
//spectrum tab
	SVAR SpectrumType								//current setting of TF spectrum popup selector
	NVAR SpectrumSV0,SpectrumSV1,SpectrumSV2	//current setting of spectrum setvariable parameters
	SVAR SpectrumPM0,SpectrumPM1,SpectrumPM2	//current setting of spectrum popupmenu parameters
	NVAR SigLevel
//appearance tab
	SVAR AmpAxis
	SVAR Norms										//current setting of normalization popup selector
	SVAR VertAxis										//current setting of vertical axis popup selector
	SVAR CTabMode									//color table
	NVAR CTabMax,CTabMin,CTabTau				//color table parameters
	NVAR NTicks										//axis tick parameters
	SVAR SwapXY										//plot rotation mode
	
//Folders
	string TFfolder
	string SettingsFolder
	string AR1Folder
	string SpectrumFolder
	string TFDecompFolder
endStructure

function TF_MakeRefs(TFgr,win)			//instead of defining lists of references in each function, a single call to this takes care of all
	struct TF_GlobalRefs &TFgr
	string win
		
	win=stringfromlist(0,win,"#")
	TFgr.TFfolder=getuserdata(win,"","TFfolder")
	
	if(datafolderexists(TFgr.TFfolder))
	//Folders
		TFgr.SettingsFolder=TFgr.TFfolder+":Settings";NewDataFolderPath(TFgr.SettingsFolder)
	
		string/g $(TFgr.SettingsFolder+":win")
		SVAR TFgr.win=$(TFgr.SettingsFolder+":win")
		TFgr.ctrlWin=TFgr.win+"#TFCtrlPanel",TFgr.dataWin=TFgr.win+"#TFWaveSelPanel"
	//global variables
		string/g $(TFgr.SettingsFolder+":dwPath"),$(TFgr.SettingsFolder+":xwPath")
		SVAR TFgr.dwPath=$(TFgr.SettingsFolder+":dwPath"), TFgr.xwPath=$(TFgr.SettingsFolder+":xwPath")
	//data selection
		string/g $(TFgr.SettingsFolder+":dwPath"),$(TFgr.SettingsFolder+":xwPath")
		SVAR TFgr.dwPath=$(TFgr.SettingsFolder+":dwPath"),TFgr.xwPath=$(TFgr.SettingsFolder+":xwPath")
	//preprocessing tab
//		string/g $(TFgr.SettingsFolder+":DetrendType")
//		SVAR TFgr.DetrendType=$(TFgr.SettingsFolder+":DetrendType")	
//		variable/g $(TFgr.SettingsFolder+":RemoveOutliers"),$(TFgr.SettingsFolder+":RemoveFliers")
//		NVAR TFgr.RemoveOutliers=$(TFgr.SettingsFolder+":RemoveOutliers"),TFgr.RemoveFliers=$(TFgr.SettingsFolder+":RemoveFliers")
//		variable/g $(TFgr.SettingsFolder+":RemoveSD"),$(TFgr.SettingsFolder+":RemoveRG")
//		NVAR TFgr.RemoveSD=$(TFgr.SettingsFolder+":RemoveSD"),TFgr.RemoveRG=$(TFgr.SettingsFolder+":RemoveRG")
	//interpolation tab
		string/g $(TFgr.SettingsFolder+":InterpType")
		SVAR TFgr.InterpType=$(TFgr.SettingsFolder+":InterpType")
		variable/g $(TFgr.SettingsFolder+":InterpRes"),$(TFgr.SettingsFolder+":LiveInterp")
		NVAR TFgr.InterpRes=$(TFgr.SettingsFolder+":InterpRes"),TFgr.LiveInterp=$(TFgr.SettingsFolder+":LiveInterp")
		variable/g $(TFgr.SettingsFolder+":Xmin"),$(TFgr.SettingsFolder+":Xmax")
		NVAR TFgr.Xmin=$(TFgr.SettingsFolder+":Xmin"),TFgr.Xmax=$(TFgr.SettingsFolder+":Xmax")
		NVAR/Z TFgr.InterpSV0=$(TFgr.SettingsFolder+":Interpolation:"+TFgr.InterpType+":SV0")
		NVAR/Z TFgr.InterpSV1=$(TFgr.SettingsFolder+":Interpolation:"+TFgr.InterpType+":SV1")
		NVAR/Z TFgr.InterpSV2=$(TFgr.SettingsFolder+":Interpolation:"+TFgr.InterpType+":SV2")
		SVAR/Z TFgr.InterpPM0=$(TFgr.SettingsFolder+":Interpolation:"+TFgr.InterpType+":PM0")
		SVAR/Z TFgr.InterpPM1=$(TFgr.SettingsFolder+":Interpolation:"+TFgr.InterpType+":PM1")
		SVAR/Z TFgr.InterpPM2=$(TFgr.SettingsFolder+":Interpolation:"+TFgr.InterpType+":PM2")
	//decomposition tab
		string/g $(TFgr.SettingsFolder+":DecompType")
		SVAR TFgr.DecompType=$(TFgr.SettingsFolder+":DecompType")
		variable/g $(TFgr.SettingsFolder+":PeriodMin"),$(TFgr.SettingsFolder+":PeriodMax")
		NVAR TFgr.PeriodMin=$(TFgr.SettingsFolder+":PeriodMin"),TFgr.PeriodMax=$(TFgr.SettingsFolder+":PeriodMax")
		variable/g $(TFgr.SettingsFolder+":freqRes")
		NVAR TFgr.freqRes=$(TFgr.SettingsFolder+":freqRes")
		NVAR/Z TFgr.TFDecompSV0=$(TFgr.SettingsFolder+":TFDecomposition:"+TFgr.DecompType+":SV0")
		NVAR/Z TFgr.TFDecompSV1=$(TFgr.SettingsFolder+":TFDecomposition:"+TFgr.DecompType+":SV1")
		NVAR/Z TFgr.TFDecompSV2=$(TFgr.SettingsFolder+":TFDecomposition:"+TFgr.DecompType+":SV2")
		SVAR/Z TFgr.TFDecompPM0=$(TFgr.SettingsFolder+":TFDecomposition:"+TFgr.DecompType+":PM0")
		SVAR/Z TFgr.TFDecompPM1=$(TFgr.SettingsFolder+":TFDecomposition:"+TFgr.DecompType+":PM1")
		SVAR/Z TFgr.TFDecompPM2=$(TFgr.SettingsFolder+":TFDecomposition:"+TFgr.DecompType+":PM2")
	//spectrum tab
		string/g $(TFgr.SettingsFolder+":SpectrumType")
		SVAR TFgr.SpectrumType=$(TFgr.SettingsFolder+":SpectrumType")
		NVAR/Z TFgr.SpectrumSV0=$(TFgr.SettingsFolder+":Spectrum:"+TFgr.SpectrumType+":SV0")
		NVAR/Z TFgr.SpectrumSV1=$(TFgr.SettingsFolder+":Spectrum:"+TFgr.SpectrumType+":SV1")
		NVAR/Z TFgr.SpectrumSV2=$(TFgr.SettingsFolder+":Spectrum:"+TFgr.SpectrumType+":SV2")
		SVAR/Z TFgr.SpectrumPM0=$(TFgr.SettingsFolder+":Spectrum:"+TFgr.SpectrumType+":PM0")
		SVAR/Z TFgr.SpectrumPM1=$(TFgr.SettingsFolder+":Spectrum:"+TFgr.SpectrumType+":PM1")
		SVAR/Z TFgr.SpectrumPM2=$(TFgr.SettingsFolder+":Spectrum:"+TFgr.SpectrumType+":PM2")
		variable/g $(TFgr.SettingsFolder+":SigLevel")
		NVAR TFgr.SigLevel=$(TFgr.SettingsFolder+":SigLevel")
	//appearance tab
		string/g $(TFgr.SettingsFolder+":AmpAxis"),$(TFgr.SettingsFolder+":VertAxis")
		SVAR TFgr.AmpAxis=$(TFgr.SettingsFolder+":AmpAxis"),TFgr.VertAxis=$(TFgr.SettingsFolder+":VertAxis")
		string/g $(TFgr.SettingsFolder+":Norms")
		SVAR TFgr.Norms=$(TFgr.SettingsFolder+":Norms")
		string/g $(TFgr.SettingsFolder+":CTabMode")
		SVAR TFgr.CTabMode=$(TFgr.SettingsFolder+":CTabMode")
		if(strlen(TFgr.CTabMode)==0)
			TFgr.CTabMode="Terrain"
		endif
		variable/g $(TFgr.SettingsFolder+":CTabMax"),$(TFgr.SettingsFolder+":CTabMin")
		NVAR TFgr.CTabMax=$(TFgr.SettingsFolder+":CTabMax"),TFgr.CTabMin=$(TFgr.SettingsFolder+":CTabMin")
		variable/g $(TFgr.SettingsFolder+":CTabTau")
		NVAR TFgr.CTabTau=$(TFgr.SettingsFolder+":CTabTau")
		variable/g $(TFgr.SettingsFolder+":NTicks")
		NVAR TFgr.NTicks=$(TFgr.SettingsFolder+":NTicks")
		string/g $(TFgr.SettingsFolder+":SwapXY")
		SVAR TFgr.SwapXY=$(TFgr.SettingsFolder+":SwapXY")

//Folders
		TFgr.AR1Folder=TFgr.TFfolder+":AR1";NewDataFolderPath(TFgr.AR1Folder)
		TFgr.SpectrumFolder=TFgr.TFfolder+":Spectrum:"+TFgr.SpectrumType;NewDataFolderPath(TFgr.SpectrumFolder)
		TFgr.TFDecompFolder=TFgr.TFfolder+":TFDecomposition:"+TFgr.DecompType
		if(!StringMatch(TFgr.DecompType,"-none-"))
			NewDataFolderPath(TFgr.TFDecompFolder)
		endif
		if(!StringMatch(TFgr.SpectrumType,"-none-"))
			NewDataFolderPath(TFgr.SpectrumFolder)
		endif
	
	//waves
	//data
		wave/Z TFgr.dw=$(TFgr.dwPath),TFgr.xw=$(TFgr.xwPath)
		wave/Z TFgr.dw_int=$(TFgr.TFfolder+":IntData")
	//decomposition
		wave/Z TFgr.TFmag=$(TFgr.TFDecompFolder+":magnitude"),TFgr.TFGlobalSpec=$(TFgr.TFDecompFolder+":GlobalSpec"),TFgr.TFBGnorm=$(TFgr.TFDecompFolder+":BGnorm")
		wave/Z TFgr.TFperiod=$(TFgr.TFDecompFolder+":period"),TFgr.TFfrequency=$(TFgr.TFDecompFolder+":frequency"),TFgr.TFscale=$(TFgr.TFDecompFolder+":scale")
		wave/Z TFgr.TFcoi=$(TFgr.TFDecompFolder+":COI")
	//spectrum
		wave/Z TFgr.AR1Mag=$(TFgr.AR1folder+":Amplitude"),TFgr.AR1period=$(TFgr.AR1folder+":period"),TFgr.AR1frequency=$(TFgr.AR1folder+":frequency")
		wave/Z TFgr.PgramMag=$(TFgr.SpectrumFolder+":Amplitude"),TFgr.PgramPh=$(TFgr.SpectrumFolder+":Phase")
		wave/Z TFgr.PgramFrequency=$(TFgr.SpectrumFolder+":Frequency"),TFgr.PgramPeriod=$(TFgr.SpectrumFolder+":Period")
	//appearance
		wave/Z TFgr.CTabMod=$(TFgr.TFFolder+":CTabMod")
		wave/Z/t TFgr.vertP=$(TFgr.TFfolder+":vertP"),TFgr.vertF=$(TFgr.TFfolder+":vertF")
		wave/Z TFgr.vertPy=$(TFgr.TFfolder+":vertPy"),TFgr.vertFy=$(TFgr.TFfolder+":vertFy")			//manual tick waves for periods
	
	//for display
		wave/Z TFgr.PlotTFdecomp=$(" "),TFgr.PlotXTFdecomp=$(" ")
		wave/Z TFgr.PlotXAR1=$(" "),TFgr.PlotXPgram=$(" ")
		strSwitch(TFgr.Norms)
			case "Magnitude":
				if(waveexists(TFgr.TFmag))
					wave TFgr.PlotTFdecomp=$getWavesDataFolder(TFgr.TFmag,2)
				endif
				break
			case "AR normalized":
				if(waveexists(TFgr.TFBGnorm))
					wave TFgr.PlotTFdecomp=$getWavesDataFolder(TFgr.TFBGnorm,2)
				endif
				break
		endSwitch
		strswitch(TFgr.VertAxis)
			case "Linear frequency":
				if(WaveExists(TFgr.AR1Frequency))
					wave TFgr.PlotXAR1=$getWavesDataFolder(TFgr.AR1Frequency,2)
				endif
				if(WaveExists(TFgr.PgramFrequency))
					wave TFgr.PlotXPgram=$getWavesDataFolder(TFgr.PgramFrequency,2)
				endif
				if(WaveExists(TFgr.TFfrequency))
					wave TFgr.plotXTFdecomp=$getWavesDataFolder(TFgr.TFfrequency,2)
				endif
				break
			case "Linear period":
			case "Log(2)":
			case "Log(10)":
				if(WaveExists(TFgr.AR1Period))
					wave TFgr.PlotXAR1=$getWavesDataFolder(TFgr.AR1Period,2)
				endif
				if(WaveExists(TFgr.PgramPeriod))
					wave TFgr.PlotXPgram=$getWavesDataFolder(TFgr.PgramPeriod,2)
				endif
				if(WaveExists(TFgr.TFperiod))
					wave TFgr.plotXTFdecomp=$getWavesDataFolder(TFgr.TFperiod,2)
				endif
				break
		endSwitch
	endif

//version check
	variable TFversion=str2num(getuserdata(win,"","TFversion"))
	if(TFversion!=kTFVersion || numtype(TFversion))
		dowindow/k $win
		TimeFrequencyInvestigation()
		TFWBListbox(TFgr.dwPath,WMWS_SelectionChanged,winname(0,1)+"#TFWaveSelPanel","DataWaveSelect")
		abort "Time frequency procedures version update!"
	endif
end

function TF_LoadSettings(TFgr,dwPath,pan)
	struct TF_GlobalRefs &TFgr
	string dwPath,pan
	
	
	SVAR/Z newwin=$(dwPath+"_TF:settings:win")
	wave dw=$dwPath
	string win=stringfromlist(0,pan,"#")
	
	if(waveexists(dw))					//Check whether data wave is being changed
		if(SVAR_exists(newwin))		//has wave been analyzed?
			if(strlen(newwin)>0 && cmpstr(newwin,win))
				DoWindow/F $(newwin)
			else
				v_flag=0
			endif
		else
			V_flag=0
		endif
		if(V_flag)							//wave analyzed in already open window
			WS_ClearSelection(pan,"DataWaveSelect")
			if(waveexists(TFgr.dw))
				WS_SelectAnObject(pan,"DataWaveSelect",getwavesdatafolder(TFgr.dw,2),openFoldersAsNeeded=1)	//reset data wave selector to previous selection
			endif
			TF_MakeRefs(TFgr,newWin)
			doalert 0,"Wave is analyzed in a different window!"
			return 0											//proper window has already been brought to front, so should exit
		else								//need to change the settings for this window
			SetWindow $win userdata(TFfolder)=(dwPath+"_TF")	//redirect window user data TFfolder to newly selected dw
			TFgr.win=""					//detach previous dw selection from this window
			TF_MakeRefs(TFgr,win)	
			if(!datafolderexists(TFgr.TFfolder))		//never before analyzed
				newdatafolderpath(dwPath+"_TF")
				TF_MakeRefs(TFgr,win)
				TFgr.win=win									//attach newly selected dw to this window
				TF_MakeRefs(TFgr,win)						//probably only necessary to regenerate window names
			//Control states
				TFgr.dwPath=dwPath
				TF_SetParameterDefaults(TFgr,1)
			else
				TFgr.win=win									//attach newly selected dw to this window
				TF_MakeRefs(TFgr,win)						//probably only necessary to regenerate window names
			endif
			TF_BuildControlPanel(TFgr)

			//Control states
			if(waveexists(TFgr.dw))
				WS_ClearSelection(pan,"DataWaveSelect")
				WS_SelectAnObject(pan,"DataWaveSelect",getwavesdatafolder(TFgr.dw,2),openFoldersAsNeeded=1)
			endif
			WS_ClearSelection(pan,"TimeWaveSelect")
			if(waveexists(TFgr.xw))
				WS_SelectAnObject(pan,"TimeWaveSelect",getwavesdatafolder(TFgr.xw,2),openFoldersAsNeeded=1)
			else
				WS_OpenAFolderFully(pan,"TimeWaveSelect",getwavesdatafolder(TFgr.dw,1))
			endif
		endif
	endif
end

Function/s TFListFunctions(prefix,none)
	string prefix
	variable none
	
	variable XY
	
	string FuncList,FuncListXY
	FuncList=(FunctionList(prefix+"_*",";","NPARAMS:1,VALTYPE:1,KIND:6"))
	FuncList=RemoveFromList(prefix+"_prototype",FuncList)
	FuncList=ReplaceString(prefix+"_",FuncList,"")
	
	
	if(XY)
		FuncListXY=(FunctionList(prefix+"XY_*",";","NPARAMS:1,VALTYPE:1,KIND:6"))
		FuncListXY=RemoveFromList(prefix+"XY_prototype",FuncListXY)
		FuncListXY=ReplaceString(prefix+"XY_",FuncListXY,"")	
		if(itemsinlist(FuncListXY))
			FuncList="\\M1(XY methods:;\\M0"+FuncListXY+"\\M1(waveform methods:;\\M0"+FuncList
		endif
	endif
	
	if(none)
		FuncList="none;\\M1-\\M0;"+FuncList
	endif	
	return FuncList
end

Function TF_SetParameterDefaults(TFgr,new)
	struct TF_GlobalRefs &TFgr
	variable new
	
	TF_MakeRefs(TFgr,TFgr.win)
	
	if(new)
	//global variables
	//data selection tab
//		TFgr.RemoveSD=3;TFgr.RemoveRG=1							//statistically "bad" parameters
	//interpolation tab
		TFgr.InterpType=stringfromlist(0,TFListFunctions("INTERPOLATION",0))
		TFgr.InterpRes=nan;TFgr.Xmin=nan;TFgr.Xmax=nan			//interpolation parameters
		TFgr.LiveInterp=1
	//decomposition tab
		TFgr.DecompType=stringfromlist(0,TFListFunctions("TFDECOMPOSITION",1))
		TFgr.PeriodMin=nan;TFgr.PeriodMax=nan						//decomposition parameters
		TFgr.freqRes=1
	//spectrum tab
		TFgr.SpectrumType=stringfromlist(0,TFListFunctions("SPECTRUM",1))
		TFgr.sigLevel=95													//significance level for spectrum display
	//appearance tab
		TFgr.VertAxis=stringfromlist(2,KPopupvalsVertAxis)
		TFgr.AmpAxis=stringfromlist(0,KPopupvalsAmpAxis)
		TFgr.Norms=stringfromlist(0,KPopupvalsNorms)
		TFgr.CTabMax=nan;TFgr.CTabMin=nan;TFgr.CTabTau=10		//color table parameters
		TFgr.CTabMode="Terrain"			
		TFgr.NTicks=5													//axis parameters
		TFgr.SwapXY=stringfromlist(0,KPopupvalsSwapXY)
	endif

	variable np=numpnts(TFgr.dw)
	if(waveexists(TFgr.xw))
		differentiate/P/EP=1/METH=1 TFgr.xw /D=$(TFgr.SettingsFolder+":tmp_dxw")
		wave tmp_dxw=$(TFgr.SettingsFolder+":tmp_dxw")
		sort tmp_dxw,tmp_dxw
		wavestats/M=1/q TFgr.xw
		TFgr.InterpRes=tmp_dxw[(v_npnts-2)/2]		//calculate median delta
		TFgr.Xmin=v_min;TFgr.Xmax=v_max
		killwaves tmp_dxw
	elseif(waveexists(TFgr.dw))
		TFgr.InterpRes=deltax(TFgr.dw)			//use data deltax
		if(TFgr.InterpRes>0)
			TFgr.Xmin=leftx(TFgr.dw);TFgr.Xmax=rightx(TFgr.dw)-TFgr.InterpRes
		else
			TFgr.Xmax=leftx(TFgr.dw);TFgr.Xmin=rightx(TFgr.dw)-TFgr.InterpRes
		endif
	endif
	TFgr.InterpRes=abs(TFgr.InterpRes)
	TFgr.PeriodMin=2*TFgr.InterpRes;TFgr.PeriodMax=(TFgr.Xmax-TFgr.Xmin)/4

	TF_BuildControlPanel(TFgr)
	WS_ClearSelection(TFgr.dataWin,"DataWaveSelect")
	WS_SelectAnObject(TFgr.dataWin,"DataWaveSelect",getwavesdatafolder(TFgr.dw,2),openFoldersAsNeeded=1)
	WS_ClearSelection(TFgr.dataWin,"TimeWaveSelect")
	if(waveexists(TFgr.xw))
		WS_SelectAnObject(TFgr.dataWin,"TimeWaveSelect",getwavesdatafolder(TFgr.xw,2),openFoldersAsNeeded=1)
	else
				WS_OpenAFolderFully(TFgr.dataWin,"TimeWaveSelect",getwavesdatafolder(TFgr.dw,1))
	endif
end

Function TF_WBLNameFilter(aName,contents)
	string aName;variable contents
	
	if(stringMatch(lowerstr(aName),"root:packages*"))
		return 0
	elseif(stringMatch(aName,"*_TF:*"))
		return 0
	else
		return 1
	endif
end

Function TimeFrequencyInvestigation()		//builds initial controls
	
	string win,TFCtrlPanelName,TFWaveSelPanelName
	win=uniquename("TimeFrequency",6,0)
	TFCtrlPanelName=win+"#TFCtrlPanel"
	TFWaveSelPanelName=win+"#TFWaveSelPanel"
//Build window
	Display/K=1/W=(5,44,705,605)/N=$(win) as "Time Frequency Plot"
	Modifygraph/W=$win margin(left)=72,margin(bottom)=72,margin(right)=(72)
	NewPanel/k=2/EXT=0/HOST=$win/n=TFWaveSelPanel/W=(5,425,250,425) as "Select waves:"
	NewPanel/k=2/EXT=2/HOST=$win/n=TFCtrlPanel/W=(700,0,700,150)/HIDE=1 as "Time Frequency controls"
	SetWindow $win userdata(TFversion)=num2str(kTFVersion)
	
//Build controls for data selection - other controls will appear only when data wave has been selected
	WaveBrowserListbox("DataWaveSelect",TFWaveSelPanelName,promptText="Select Data Wave:",Xpos=5,Ypos=5,Xsize=240,Ysize=190,WaveOpts="CMPLX:0,TEXT:0,INTEGER:0,DIMS:1,UNSIGNED:0,WORD:0",procName="TFWBListbox",nameFilterProc="TF_WBLNameFilter")
	PopupMenu sortKind pos={5,400},title="Sort by",win=$TFWaveSelPanelName
	MakePopupIntoWaveSelectorSort(TFWaveSelPanelName,"DataWaveSelect","sortKind")
	
//help and wave selection button
	ControlBar 25
	Button DisplayHelp pos={5,2},size={60,20},proc=TFButton,title="Help",win=$win
	Button DisplayWaveSelectors pos={75,2},size={150,20},proc=TFButton,title="Hide Wave Selection",win=$win
	
	SetWindow $win userdata(TFfolder)=(" ")		//because of bug: datafolderexists("") returns 1
end	

static function WaveBrowserListbox(controlName,windowName[,promptText,Xpos,Ypos,Xsize,Ysize,WaveOpts,procName,add_calculated,nameFilterProc])
	string controlName,windowName
	variable Xpos,Ypos,Xsize,Ysize
	string WaveOpts,promptText,procName
	variable add_calculated
	string nameFilterProc
	
	string listBoxFolder
	
	if(paramisdefault(Xpos) || paramisdefault(Ypos))
		Xpos=5;Ypos=5
	endif
	if(paramisdefault(Xsize) || paramisdefault(Ysize))
		Xsize=100;Ysize=100
	endif
	if(paramisdefault(WaveOpts))
		waveOpts=""
	endif
	if(paramisdefault(nameFilterProc))
		nameFilterProc=""
	endif
	if(paramisdefault(PromptText))
		PromptText=""
	else
		TitleBox $(stringfromlist(0,controlName,"_")+"Title_"+stringfromlist(1,controlName,"_")),pos={Xpos,Ypos},size={Xsize,Ysize},title=promptText,fstyle=2^0,win=$windowName
		Ypos+=18;Ysize-=18
	endif
	ListBox $controlName,pos={Xpos,Ypos},size={Xsize,Ysize},widths={300},win=$windowName

	MakeListIntoWaveSelector(windowName,controlName,content=WMWS_Waves,selectionMode=WMWS_SelectionSingle,listoptions=WaveOpts,nameFilterProc=nameFilterProc)
	if(!paramisdefault(procName))
		WS_SetNotificationProc(windowName, controlName, procname,isExtendedProc=1)
	endif
	if(add_calculated)
		WS_AddSelectableString(windowName,controlName,"_calculated_")
	endif
end

Function TF_BuildControlPanel(TFgr)
	struct TF_GlobalRefs &TFgr
	
	variable np=numpnts(TFgr.dw)
	string WaveOpts	
	
//***Data selection controls
	sprintf WaveOpts,"CMPLX:0,TEXT:0,INTEGER:0,DIMS:1,UNSIGNED:0,WORD:0,MAXROWS:%u,MINROWS:%u",np,np
	WaveBrowserListbox("TimeWaveSelect",TFgr.dataWin,promptText="For XY Data, Select X Wave:",Xpos=5,Ypos=205,Xsize=240,Ysize=190,WaveOpts=WaveOpts,procName="TFWBListbox",add_calculated=1,nameFilterProc="TF_WBLNameFilter")
	MakePopupIntoWaveSelectorSort(TFgr.dataWin,"TimeWaveSelect","sortKind")
	
//***unhide tabbed control panel
	SetWindow $TFgr.ctrlWin,hide=0
//Set to correct tab
	struct WMTabControlAction TCs
	TCs.win=TFgr.ctrlWin
	TCs.ctrlName="Tabs"
	controlInfo/W=$TCs.win Tabs
	TCs.tab=V_value
	TCs.eventcode=2
	TFTabs(TCs)
end

Function TFAddTab_prototype(TFgr)
	struct TF_GlobalRefs &TFgr
	
end

//Function TFAddTab_Preprocessing(TFgr)
//	struct TF_GlobalRefs &TFgr
////***Preprocessing controls
//	PopupMenu DetrendType pos={5,25},proc=TFPopup,title="Detrend:",value=TFListFunctions("DETREND",0),mode=(max(0,WhichListItem(TFgr.DetrendType,TFListFunctions("DETREND",1)))+1),win=$TFgr.ctrlWin
//	CheckBox RemoveFliers pos={5,50},size={100,20},proc=TFCheckBox,title="Remove fliers",variable=TFgr.RemoveFliers,win=$TFgr.ctrlWin
//	CheckBox RemoveOutliers pos={5,75},size={100,20},proc=TFCheckBox,title="Remove outliers",variable=TFgr.RemoveOutliers,win=$TFgr.ctrlWin
//end
//
Function TFAddTab_Interpolation(TFgr)
	struct TF_GlobalRefs &TFgr
//***Interpolation controls
	PopupMenu InterpType pos={5,25},proc=TFPopup,title="Interpolation:",value=TFListFunctions("INTERPOLATION",0),mode=(max(0,WhichListItem(TFgr.InterpType,TFListFunctions("INTERPOLATION",0)))+1),win=$TFgr.ctrlWin
	SetVariable InterpRes pos={5,50},size={175,20},bodywidth=70,limits={sigdigits(TFgr.InterpRes,1)/10,inf,sigdigits(TFgr.InterpRes,1)/10},proc=TFSetVariable,title="Interp. Resolution:",value=$(TFgr.SettingsFolder+":InterpRes"),win=$TFgr.ctrlWin
	SetVariable XMin pos={5,75},size={175,20},bodywidth=70,limits={-inf,inf,0},proc=TFSetVariable,title="Minimum X Value:",value=$(TFgr.SettingsFolder+":XMin"),win=$TFgr.ctrlWin
	SetVariable XMax pos={5,100},size={175,20},bodywidth=70,limits={-inf,inf,0},proc=TFSetVariable,title="Maximum X Value:",value=$(TFgr.SettingsFolder+":XMax"),win=$TFgr.ctrlWin
	CheckBox LiveInterp pos={600,25},size={100,20},proc=TFCheckBox,title="Live Update",variable=TFgr.LiveInterp,win=$TFgr.ctrlWin
	Button ResetValues pos={600,50},size={100,20},proc=TFButton,title="Reset Values",win=$TFgr.ctrlWin
	Button DoInterpolate pos={600,75},size={100,20},proc=TFButton,title="Re-interpolate",win=$TFgr.ctrlWin
	TF_InterpOptions(TFgr)
end

Function TFAddTab_Decomposition(TFgr)
	struct TF_GlobalRefs &TFgr
//***TF decomposition controls
	PopupMenu DecompType pos={5,25},proc=TFPopup,title="Decomposition:",value=TFListFunctions("TFDECOMPOSITION",1),mode=(max(0,WhichListItem(TFgr.DecompType,TFListFunctions("TFDECOMPOSITION",1)))+1),win=$TFgr.ctrlWin
//	Button Decompose pos={500,50},size={75,20},proc=TFButton,title="Calculate",win=$TFgr.ctrlWin
	SetVariable PeriodMin pos={5,50},size={175,20},bodywidth=70,limits={TFgr.interpRes*2,inf,sigdigits(TFgr.PeriodMin,1)/10},proc=TFSetVariable,title="Minimum Period:",value=$(TFgr.SettingsFolder+":PeriodMin"),win=$TFgr.ctrlWin
	SetVariable PeriodMax pos={5,75},size={175,20},bodywidth=70,limits={TFgr.interpRes*2,inf,sigdigits(TFgr.PeriodMax,1)/10},proc=TFSetVariable,title="Maximum Period:",value=$(TFgr.SettingsFolder+":PeriodMax"),win=$TFgr.ctrlWin
//	SetVariable freqRes pos={5,100},size={175,20},bodywidth=70,limits={1,inf,0.5},proc=TFSetVariable,title="spectral resolution multiple:",value=$(TFgr.SettingsFolder+":freqRes"),win=$TFgr.ctrlWin
	TF_DecompOptions(TFgr)
end

Function TFAddTab_Spectrum(TFgr)
	struct TF_GlobalRefs &TFgr
//***Spectrum controls
	PopupMenu SpectrumType pos={5,25},proc=TFPopup,title="Spectrum",value=TFListFunctions("SPECTRUM",1),mode=(max(0,WhichListItem(TFgr.SpectrumType,TFListFunctions("SPECTRUM",1)))+1),win=$TFgr.ctrlWin
	SetVariable PeriodMin pos={5,50},size={175,20},bodywidth=70,limits={TFgr.interpRes*2,inf,sigdigits(TFgr.PeriodMin,1)/10},proc=TFSetVariable,title="Minimum Period:",value=$(TFgr.SettingsFolder+":PeriodMin"),win=$TFgr.ctrlWin
	SetVariable PeriodMax pos={5,75},size={175,20},bodywidth=70,limits={TFgr.interpRes*2,inf,sigdigits(TFgr.PeriodMax,1)/10},proc=TFSetVariable,title="Maximum Period:",value=$(TFgr.SettingsFolder+":PeriodMax"),win=$TFgr.ctrlWin
	SetVariable freqRes pos={5,100},size={175,20},bodywidth=70,limits={1,inf,0.5},proc=TFSetVariable,title="0 pad (Multiple of N):",value=$(TFgr.SettingsFolder+":freqRes"),win=$TFgr.ctrlWin
	SetVariable SigLevel pos={5,125},size={175,20},bodywidth=70,limits={0,100,5},proc=TFSetVariable,title="Significance Level:",value=$(TFgr.SettingsFolder+":SigLevel"),win=$TFgr.ctrlWin	
	TF_SpectrumOptions(TFgr)
end

Function TFAddTab_Appearance(TFgr)
	struct TF_GlobalRefs &TFgr
//***Appearance controls
	PopupMenu Norms pos={5,25},size={200,20},proc=TFPopup,title="Amplitude Normalization:",value=KPopupvalsNorms,mode=(max(0,WhichListItem(TFgr.Norms,KPopupvalsNorms))+1),win=$TFgr.ctrlWin
	PopupMenu AmpAxis pos={5,50},size={200,20},proc=TFPopup,title="Spectrum Amplitude Axis:",value=KPopupvalsAmpAxis,mode=(max(0,WhichListItem(TFgr.AmpAxis,KPopupvalsAmpAxis))+1),win=$TFgr.ctrlWin
	PopupMenu VertAxis pos={5,75},size={200,20},proc=TFPopup,title="Period/Frequency Axes:",value=KPopupvalsVertAxis,mode=(max(0,WhichListItem(TFgr.VertAxis,KPopupvalsVertAxis))+1),win=$TFgr.ctrlWin
	SetVariable ReciprocalTicks pos={5,100},size={200,20},bodywidth=40,limits={1,inf,1},proc=TFSetVariable,title="Approximate Number of Ticks:",value=$(TFgr.SettingsFolder+":NTicks"),win=$TFgr.ctrlWin
	PopupMenu CTabMode pos={250,25},size={200,20},proc=TFPopup,title="Color Table",value="*COLORTABLEPOP*",mode=(whichlistitem(TFgr.CTabMode,CTabList())+1),win=$TFgr.ctrlWin
	SetVariable CTabTau pos={250,50},size={200,20},bodywidth=45,limits={-inf,inf,1},proc=TFSetVariable,title="Color Table Nonlinearity:",value=$(TFgr.SettingsFolder+":CTabTau"),win=$TFgr.ctrlWin
	SetVariable CTabMin pos={250,75},size={200,20},bodywidth=60,limits={0,inf,0.1},proc=TFSetVariable,title="Minimum Color:",format="%3.3G",value=$(TFgr.SettingsFolder+":CTabMin"),win=$TFgr.ctrlWin
	SetVariable CTabMax pos={250,100},size={200,20},bodywidth=60,limits={0,inf,0.1},proc=TFSetVariable,title="Maximum Color:",format="%3.3G",value=$(TFgr.SettingsFolder+":CTabMax"),win=$TFgr.ctrlWin
	PopupMenu SwapXY pos={540,75},proc=TFPopup,title="",value=KPopupvalsSwapXY,mode=(max(0,WhichListItem(TFgr.SwapXY,KPopupvalsSwapXY))+1),win=$TFgr.ctrlWin	
	Button CopyPlot pos={550,125},size={120,20},proc=TFButton,title="Make Plot Copy",win=$TFgr.ctrlWin
end

Function TFCheckBox(CBS) : CheckBoxControl
	struct WMCheckboxAction &CBS
	
	if(CBS.eventCode==2)		
		struct TF_GlobalRefs TFgr
		TF_MakeRefs(TFgr,CBS.win)
		
		StrSwitch(CBs.CtrlName)
			case "LiveInterp":
				break
//			case "RemoveFliers":
//			case "RemoveOutliers":
//				if(TFgr.RemoveFliers || TFgr.RemoveOutliers)
//					SetVariable RemoveSD bodywidth=60,size={150,20},pos={420,50},limits={0,10,0.1},proc=TFSetVariable,title="cutoff (sigma):",value=$(TFgr.SettingsFolder+":RemoveSD"),win=$CBS.win
//					SetVariable RemoveRG bodywidth=60,size={150,20},pos={420,75},limits={0,inf,0.1},proc=TFSetVariable,title="range:",value=$(TFgr.SettingsFolder+":RemoveRG"),win=$CBS.win
//				else
//					KillControl/w=$CBS.win RemoveSD
//					KillControl/w=$CBS.win RemoveRG
//				endif
//				break
		endSwitch

		if(TFgr.LiveInterp)						//if live interpolation is selected, reinterpolate
			TF_InitiateInterpolation(TFgr)
			TF_UpdatePlot(TFgr)
		endif
	endif
end

Function TFSetVariable(SVs) : SetVariableControl
	STRUCT WMSetVariableAction &SVs
	
	if(SVS.eventcode==2 || SVS.eventcode==1)		
		struct TF_GlobalRefs TFgr
		TF_MakeRefs(TFgr,SVS.win)
		
		StrSwitch(SVs.CtrlName)
			case "InterpRes":
				TFgr.PeriodMin=SVS.dval*2					//adjust sampling interval in interpolated data
				SetVariable InterpRes limits={sigdigits(TFgr.InterpRes,1)/10,inf,sigdigits(TFgr.InterpRes,1)/10},win=$SVS.win
			case "PeriodMin":
			case "PeriodMax":
				SetVariable PeriodMin limits={TFgr.interpRes*2,inf,sigdigits(TFgr.PeriodMin,1)/10},win=$SVS.win
				SetVariable PeriodMax limits={TFgr.interpRes*2,inf,sigdigits(TFgr.PeriodMax,1)/10},win=$SVS.win
				SetVariable PeriodMin limits={TFgr.interpRes*2,inf,sigdigits(TFgr.PeriodMin,1)/10},win=$SVS.win
				SetVariable PeriodMax limits={TFgr.interpRes*2,inf,sigdigits(TFgr.PeriodMax,1)/10},win=$SVS.win
			case "freqRes":
			case "Xmin":
			case "Xmax":
			case "RemoveSD":
			case "RemoveRG":
				if(TFgr.LiveInterp)
					TF_InitiateInterpolation(TFgr)							//reinterpolate if live interpolation is selected
				endif
				TF_initiateSpectrum(TFgr)
				break
			case "InterpSV1":
			case "InterpSV2":
			case "InterpSV3":
				TF_initiateInterpolation(TFgr)
				break
			case "TFDecompSV1":
			case "TFDecompSV2":
			case "TFDecompSV3":
				TF_initiateDecomposition(TFgr)
				break
			case "SpectrumSV1":
			case "SpectrumSV2":
			case "SpectrumSV3":
				TF_initiateSpectrum(TFgr)
				break
			case "SigLevel":
				TF_AR1(TFgr)
				break
			case "CTabTau":
				TF_LogColorTable(TFgr)			//adjusts nonlinearity of color table
				break
			case "CTabMax":
			case "CTabMin":
				break
			case "PTickSpace":
			case "FTickSpace":
			case "LabelSpace":
				break
		EndSwitch
		TF_UpdatePlot(TFgr)
	endif	
End

Function TF_LogColorTable(TFgr)
	struct TF_GlobalRefs &TFgr
	
	variable n=100
	make/o/n=(n) $(TFgr.TFFolder+":CTabMod")
	wave TFgr.CTabMod=$(TFgr.TFFolder+":CTabMod")
	
	TFgr.CTabMod[]=(abs(TFgr.CTabTau)+1.00001)^(p/n)-1
	TFgr.CTabMod[]/=TFgr.CTabMod[n-1]
	if(TFgr.CTabTau<0)
		TFgr.CTabMod=-(TFgr.CTabMod)+1
		reverse/P TFgr.CTabMod
	endif
end

Function TFWBListbox(SelectedItem,EventCode,win,ControlName)
	string SelectedItem,win,ControlName
	variable EventCode
	
	if(EventCode==WMWS_SelectionChanged)

		struct TF_GlobalRefs TFgr
		TF_MakeRefs(TFgr,win)
		
		strSwitch(ControlName)
			case "DataWaveSelect":
				if(SVAR_exists(TFgr.dwPath))
					if(stringmatch(selectedItem,TFgr.dwPath))
						break
					endif
				endif
				TF_LoadSettings(TFgr,selectedItem,win)
				break
			case "TimeWaveSelect":
				if(!waveexists($selectedItem))
					selectedItem=""
				endif
				if(SVAR_exists(TFgr.xwPath))
					if(stringmatch(selectedItem,TFgr.xwPath))
						break
					endif
				endif
				wave/z TFgr.xw=$selectedItem
				if(waveexists(TFgr.xw))
					TFgr.xwPath=getWavesDataFolder(TFgr.xw,2)
				else
					TFgr.xwPath=""
				endif
				TF_MakeRefs(TFgr,TFgr.win)
				TF_SetParameterDefaults(TFgr,1)
				break
		endSwitch
		
		if(TFgr.liveInterp)
			TF_InitiateInterpolation(TFgr)
		endif
		TF_UpdatePlot(TFgr)
	endif
end

Function TFPopup(PUs)
	struct WMPopupAction &PUs
	
	if(PUs.eventCode==2)
		struct TF_GlobalRefs TFgr
		TF_MakeRefs(TFgr,PUs.win)
	
		StrSwitch(PUs.ctrlName)
			case "InterpType":								//change in interpolation method
				TFgr.InterpType=PUs.popStr
				TF_InterpOptions(TFgr)
				if(TFgr.LiveInterp)								//reinterpolate if live interpolation is selected
					TF_InitiateInterpolation(TFgr)
				endif
				break
			case "InterpPM1":
				TFgr.InterpPM0=PUs.popStr
				TF_InitiateInterpolation(TFgr)
				break
			case "InterpPM2":
				TFgr.InterpPM1=PUs.popStr
				TF_InitiateInterpolation(TFgr)
				break
			case "InterpPM3":
				TFgr.InterpPM2=PUs.popStr
				TF_InitiateInterpolation(TFgr)
				break
			case "DecompType":									//change in time-frequency decomposition
				TFgr.DecompType=PUs.popStr
				TF_MakeRefs(TFgr,TFgr.win)
	//			TFgr.CTabMax=nan;TFgr.CTabMin=nan											//this forces a recalculation of the color table
				TF_DecompOptions(TFgr)
				TF_InitiateDecomposition(TFgr)
				break
			case "TFDecompPM1":
				TFgr.TFDecompPM0=PUs.popStr
				TF_InitiateDecomposition(TFgr)
				break
			case "TFDecompPM2":
				TFgr.TFDecompPM1=PUs.popStr
				TF_InitiateDecomposition(TFgr)
				break
			case "TFDecompPM3":
				TFgr.TFDecompPM2=PUs.popStr
				TF_InitiateDecomposition(TFgr)
				break
			case "SpectrumType":				//change in spectrum decomposition - automatically recalculates spectrum
				TFgr.SpectrumType=PUs.popStr
				TF_MakeRefs(TFgr,TFgr.win)
				TF_SpectrumOptions(TFgr)
				TF_InitiateSpectrum(TFgr)
				break
			case "SpectrumPM1":
				TFgr.SpectrumPM0=PUs.popStr
				TF_InitiateSpectrum(TFgr)
				break
			case "SpectrumPM2":
				TFgr.SpectrumPM1=PUs.popStr
				TF_InitiateSpectrum(TFgr)
				break
			case "SpectrumPM3":
				TFgr.SpectrumPM2=PUs.popStr
				TF_InitiateSpectrum(TFgr)
				break
			case "VertAxis":				//change in (non)linearity of period and frequency axes
				TFgr.VertAxis=PUs.popStr
				TF_MakeRefs(TFgr,TFgr.win)
				break
			case "AmpAxis":			//change from/to log scale for spectrum amplitude
				TFgr.AmpAxis=PUs.popStr
				ModifyGraph log(Amplitude)=whichlistitem(TFgr.AmpAxis,KPopupvalsAmpAxis)
				break
			case "CTabMode":
				TFgr.CTabMode=PUs.popStr
				break
			case "Norms":				//change in normalization of time-frequency amplitudes
				TFgr.Norms=PUs.popStr
				TFgr.CTabMax=nan;TFgr.CTabMin=nan		//force change in color table limits
				break
			case "SwapXY":
				TFgr.SwapXY=PUs.popStr
				break
		EndSwitch
		TF_UpdatePlot(TFgr)
	endif
End

Function TFTabs(TCs)
	struct WMTabControlAction &TCs

//This is inefficient in processing power, but efficient in lines of code. I can't imagine a situation in which the delay due to extra processing would be noticed.

	if(TCs.eventcode==2)
		struct TF_GlobalRefs TFgr
		TF_MakeRefs(TFgr,TCs.win)
		
//gathers available tab functions, finds selected tab in new tab list
		string TabList=FunctionList("TFAddTab_*",";","NPARAMS:1,VALTYPE:1,KIND:6,WIN:Time-Frequency.ipf")	//load from this procedure file first
		string oldTabList=""
		TabList+=removefromlist(TabList,FunctionList("TFAddTab_*",";","NPARAMS:1,VALTYPE:1,KIND:6"))		//load from other procedure files
		TabList=ReplaceString("TFAddTab_",removefromlist("TFAddTab_prototype",TabList),"")
		controlInfo/W=$TFgr.ctrlWin $TCs.ctrlName
		if(v_flag)
			oldTabList=GetUserData(TFgr.ctrlWin,TCs.ctrlName,"TabList")
		endif
		string selectedTab=stringfromlist(TCs.tab,oldTabList)
		TCs.tab=max(0,whichListItem(selectedTab,TabList))
		selectedTab=stringfromlist(TCs.tab,TabList)		//repeat in case selected tab is no longer available

//kill current controls, including tab control to make sure all defunct tabs are removed
		String ControlList=ControlNameList(TCs.win)
		variable i,	nitems=itemsinlist(ControlList)
		for(i=0;i<itemsinlist(ControlList);i+=1)
			KillControl/W=$TFgr.ctrlWin $stringfromlist(i,ControlList)
		endfor
		
//Tabs
		TabControl Tabs,pos={0,0},size={700,150},proc=TFTabs,win=$TFgr.ctrlWin
		nitems=itemsinlist(TabList)
		for(i=0;i<nitems;i+=1)
			TabControl Tabs,tablabel(i)=stringfromlist(i,TabList),win=$TFgr.ctrlWin
		endfor
		TabControl $TCs.ctrlName userdata(TabList)=TabList,value=TCs.tab,win=$TFgr.ctrlWin

//build controls for current tab
		FuncRef TFAddTab_prototype TabFunc=$("TFAddTab_"+selectedTab)
		TabFunc(TFgr)
	endif
End

Function TFButton(BS) : ButtonControl
	STRUCT WMButtonAction &BS
		
	if(BS.eventcode==2)
		struct TF_GlobalRefs TFgr
		TF_MakeRefs(TFgr,BS.win)
		
		strswitch(BS.ctrlName)
			case "DisplayWaveSelectors":
				TFgr.dataWin=(BS.win+"#TFWaveSelPanel")
				getwindow $TFgr.dataWin hide
				if(v_value)
					Button DisplayWaveSelectors title="Hide Wave Selection",win=$BS.win
					setwindow $TFgr.dataWin,hide=0
				else
					Button DisplayWaveSelectors title="Show Wave Selection",win=$BS.win
					setwindow $TFgr.dataWin,hide=1				
				endif
				break
			case "DisplayHelp":		//displays help file at subtopic for current tab
				TFgr.dataWin=(BS.win+"#TFWaveSelPanel")
				TFgr.ctrlWin=(BS.win+"#TFCtrlPanel")
				string helpTopic="Using the Time Frequency plot"
				getwindow $TFgr.ctrlWin hide
				if(!v_value)
					controlInfo/W=$TFgr.ctrlWin Tabs
					helpTopic+="["+S_Value+"]"
				endif
				DisplayHelpTopic/k=1 helpTopic
				break
			case "Decompose":		//do time-frequency decomposition
				TF_InitiateDecomposition(TFgr)
				TF_UpdatePlot(TFgr)
				break
			case "CopyPlot":			//duplicates the current plot area of the window, so that it can be edited, compared, etc.
				TF_UpdatePlot(TFgr,dup=1)
				break
			case "ResetValues":		//reset parameters to defaults based on data
				TF_SetParameterDefaults(TFgr,0)
				if(TFgr.LiveInterp)
					TF_InitiateInterpolation(TFgr)
				endif
				TF_UpdatePlot(TFgr)
				break
			case "DoInterpolate":	//interpolate
				TF_InitiateInterpolation(TFgr)
				TF_UpdatePlot(TFgr)
				break
		endSwitch
	endif
End

Function TF_InterpOptions(TFgr)	//adds controls specfic to spectral estimate methods
	struct TF_GlobalRefs &TFgr
	
	struct InterpolationParameters IPs
	IPs.getParams=1		//makes spectrum function return information needed to build parameter controls
	IPs.Xmin=TFgr.Xmin;IPs.Xmax=TFgr.Xmax
	IPs.interpRes=TFgr.interpRes
	
	FuncRef INTERPOLATION_prototype InterpFunc=$("INTERPOLATION_"+TFgr.InterpType)
	InterpFunc(IPs)
	string InterpSettingsFolder=TFgr.SettingsFolder+":Interpolation:"+TFgr.InterpType
	NewDataFolderPath(InterpSettingsFolder)
	variable i
	string SVpath,PMpath,ctrlName

	for(i=0;i<3;i+=1)					//loop to build setvariable controls for spectrum parameters
		SVpath=InterpSettingsFolder+":SV"+num2istr(i)
		NVAR/Z gSV=$SVpath
		ctrlName="InterpSV"+num2istr(i+1)
		if(strlen(IPs.SV.name[i]))
			if(NVAR_exists(gSV))
				IPs.SV.value[i]=gSV
			else
				variable/g $SVpath=IPs.SV.value[i]
			endif
			SetVariable $ctrlName pos={195,25+i*25},size={150,20},bodywidth=70,proc=TFSetVariable,win=$TFgr.ctrlWin
			setvariable $ctrlName limits={IPs.SV.low[i],IPs.SV.high[i],IPs.SV.inc[i]},title=IPs.SV.name[i],value=$SVpath,win=$TFgr.ctrlWin
		else
			killvariables/z gSV
			killcontrol/W=$TFgr.ctrlWin $ctrlName
		endif
	endfor
	for(i=0;i<3;i+=1)				//loop to build popup menu controls for spectrum parameters
		PMpath=InterpSettingsFolder+":PM"+num2istr(i)
		SVAR/Z gPM=$PMpath
		string/g $(PMpath+"list")
		SVAR gPMvalue=$(PMpath+"list")
		ctrlName="InterpPM"+num2istr(i+1)
		if(strlen(IPs.PM.name[i]))
			if(SVAR_exists(gPM))
				IPs.PM.popStr[i]=gPM
			else
				string/g $PMpath=IPs.PM.popStr[i]
			endif
			gPMvalue=IPs.PM.value[i]
			PopupMenu $ctrlName,pos={355,25+i*25},proc=TFPopup,title=IPs.PM.name[i],win=$TFgr.ctrlWin
			PopupMenu $ctrlName value=#(PMpath+"list"),popvalue=IPs.PM.popStr[i],mode=1,win=$TFgr.ctrlWin
		else
			killstrings/z gPM,gPMvalue
			killcontrol/W=$TFgr.ctrlWin $ctrlName
		endif
	endfor
end

Function TF_InitiateInterpolation(TFgr)		//interpolate data using user-selected method
	struct TF_GlobalRefs &TFgr
		
	struct InterpolationParameters IPs
	IPs.getParams=0					//setting to calculate spectrum
	FuncRef INTERPOLATION_prototype InterpFunc=$("INTERPOLATION_"+TFgr.InterpType)
	string InterpSettingsFolder=TFgr.SettingsFolder+":Interpolation:"+TFgr.InterpType
	NewDataFolderPath(InterpSettingsFolder)
	variable i
	string SVpath,PMpath,ctrlName

	for(i=0;i<3;i+=1)		//loop to get values of setvariable parameters
		SVpath=InterpSettingsFolder+":SV"+num2istr(i)
		NVAR/Z gSV=$SVpath
		if(NVAR_exists(gSV))
			IPs.SV.value[i]=gSV
		else
			IPs.SV.value[i]=nan
		endif
	endfor
	for(i=0;i<3;i+=1)		//loop to get values of popup menu parameters
		PMpath=InterpSettingsFolder+":PM"+num2istr(i)
		SVAR/Z gPM=$PMpath
		SVAR/Z gPMvalue=$(PMpath+"list")
		if(SVAR_exists(gPM))
			IPs.PM.popStr[i]=gPM
			IPs.PM.popNum[i]=whichlistitem(gPM,gPMvalue)+1
		else
			IPs.PM.popStr[i]=""
			IPs.PM.popNum[i]=nan
		endif
	endfor
	
//(re)create interpolated data wave
	string IntName=TFgr.TFfolder+":IntData"
	make/o/d $IntName;wave TFgr.dw_int=$IntName

//the following code duplicates the original data and x waves to avoid altering them during pre-processing and interpolation
	string ProcName=TFgr.SettingsFolder+":ProcData"
	string ProcXName=TFgr.SettingsFolder+":ProcX"
	variable np,maxpt,minpt
	duplicate/o TFgr.dw,$ProcName
	wave dw=$ProcName
	if(waveexists(TFgr.xw))			//if there is an x-value wave; references the plotted xw in case the data is being tuned
		duplicate/o TFgr.xw,$ProcXName
		wave xw=$ProcXName
	else
		duplicate/o TFgr.dw,$ProcXName	//creates dummy x-value wave with values set to x-scaling of dw
		wave xw=$ProcXName
		xw=x
	endif
//removes nan in both xw and dw
	sort dw,xw,dw
	wavestats/q/m=0 dw
	redimension/n=(v_npnts) xw,dw
	sort xw,xw,dw					//put in x-value order
	wavestats/q/m=0 xw
	redimension/n=(v_npnts) xw,dw

//truncate data to user-selected range
	minpt=max(0,binarysearch(xw,TFgr.Xmin));maxpt=min(numpnts(dw)-1,binarysearch(xw,TFgr.Xmax)+1)		//locate user-selected end-points
	if(maxpt<=0)
		maxpt=numpnts(dw)-1
	endif
	redimension/n=(maxpt+1) xw,dw
	deletepoints 0,minpt,xw,dw
	
	TFgr.Xmin=max(TFgr.Xmin,xw[0])//set xmin and xmax to values closest to user selected values
	TFgr.Xmax=min(TFgr.Xmax,xw[numpnts(xw)-1])
	
//set scaling for interpolated data wave
	np=trunc(abs((TFgr.Xmax-TFgr.Xmin)/(TFgr.InterpRes))+1)		//calculate number of points in interpolated wave
	np-=mod(np,2)													//make sure it's a multiple of 2, otherwise FFT fails
	redimension/n=(np) TFgr.dw_int
	setscale/p x TFgr.Xmin,TFgr.InterpRes,TFgr.dw_int
	
////remove fliers and outliers
//	if(TFgr.RemoveFliers)						//user wants fliers removed
//		RemoveFliers(TFgr.RemoveRG,TFgr.RemoveSD,dw,xw=xw)
//	endif
//	if(TFgr.RemoveOutliers)						//user wants outliers removed
//		RemoveOutliers(TFgr.RemoveRG,TFgr.RemoveSD,dw,xw=xw)
//	endif
	
	IPs.Xmin=TFgr.Xmin;IPs.Xmax=TFgr.Xmax
	IPs.interpRes=TFgr.interpRes
	
	wave IPs.dw_int=$getwavesdatafolder(TFgr.dw_int,2)
	wave IPs.dw=$getwavesdatafolder(dw,2)
	wave IPs.xw=$getwavesdatafolder(xw,2)
	
	InterpFunc(IPs)		//calls spectrum calculation wrapper function
	
	ReplaceNaN(TFgr.dw_int,0,1)	//interpolated waves may have NaNs where there are not enough data points - this inserts flat intervals (spectrally relatively neutral) for NaNs
	killwaves/z xw,dw
end

Function TF_DecompOptions(TFgr)	//adds controls specfic to spectral estimate methods
	struct TF_GlobalRefs &TFgr
	
	struct TFDecompParameters TFPs
	TFPs.getParams=1		//makes spectrum function return information needed to build parameter controls
	FuncRef TFDECOMPOSITION_prototype DecompFunc=$("TFDECOMPOSITION_"+TFgr.DecompType)
	DecompFunc(TFPs)
	string DecompSettingsFolder=TFgr.SettingsFolder+":TFDecomposition:"+TFgr.DecompType
	NewDataFolderPath(DecompSettingsFolder)
	variable i
	string SVpath,PMpath,ctrlName

	for(i=0;i<3;i+=1)					//loop to build setvariable controls for spectrum parameters
		SVpath=DecompSettingsFolder+":SV"+num2istr(i)
		NVAR/Z gSV=$SVpath
		ctrlName="TFDecompSV"+num2istr(i+1)
		if(strlen(TFPs.SV.name[i]))
			if(NVAR_exists(gSV))
				TFPs.SV.value[i]=gSV
			else
				variable/g $SVpath=TFPs.SV.value[i]
			endif
			SetVariable $ctrlName pos={255,25+i*25},size={150,20},bodywidth=70,proc=TFSetVariable,win=$TFgr.ctrlWin
			setvariable $ctrlName limits={TFPs.SV.low[i],TFPs.SV.high[i],TFPs.SV.inc[i]},title=TFPs.SV.name[i],value=$SVpath,win=$TFgr.ctrlWin
		else
			killvariables/z gSV
			killcontrol/W=$TFgr.ctrlWin $ctrlName
		endif
	endfor
	for(i=0;i<3;i+=1)				//loop to build popup menu controls for spectrum parameters
		PMpath=DecompSettingsFolder+":PM"+num2istr(i)
		SVAR/Z gPM=$PMpath
		string/g $(PMpath+"list")
		SVAR gPMvalue=$(PMpath+"list")
		ctrlName="TFDecompPM"+num2istr(i+1)
		if(strlen(TFPs.PM.name[i]))
			if(SVAR_exists(gPM))
				TFPs.PM.popStr[i]=gPM
			else
				string/g $PMpath=TFPs.PM.popStr[i]
			endif
			gPMvalue=TFPs.PM.value[i]
			PopupMenu $ctrlName,pos={505,25+i*25},proc=TFPopup,title=TFPs.PM.name[i],win=$TFgr.ctrlWin
			PopupMenu $ctrlName value=#(PMpath+"list"),popvalue=TFPs.PM.popStr[i],mode=1,win=$TFgr.ctrlWin
		else
			killstrings/z gPM,gPMvalue
			killcontrol/W=$TFgr.ctrlWin $ctrlName
		endif
	endfor
end

Function TF_InitiateDecomposition(TFgr)	//calculate time-frequency spectrum
	struct TF_GlobalRefs &TFgr
	
	variable MemSize

//	TFgr.CTabMax=nan;TFgr.CTabMin=nan	//forces recalculation of color table
	TF_AR1(TFgr)					//calculate expected AR1 red noise spectrum

	struct TFDecompParameters TFPs
	TFPs.getParams=0		//makes decomposition function do the calculation
	FuncRef TFDECOMPOSITION_prototype DecompFunc=$("TFDECOMPOSITION_"+TFgr.DecompType)

	string DecompSettingsFolder=TFgr.SettingsFolder+":TFDecomposition:"+TFgr.DecompType
	NewDataFolderPath(DecompSettingsFolder)
	variable i
	string SVpath,PMpath,ctrlName

	for(i=0;i<3;i+=1)					//loop to build setvariable controls for spectrum parameters
		SVpath=DecompSettingsFolder+":SV"+num2istr(i)
		NVAR/Z gSV=$SVpath
		if(NVAR_exists(gSV))
			TFPs.SV.value[i]=gSV
		else
			TFPs.SV.value[i]=nan
		endif
	endfor
	for(i=0;i<3;i+=1)				//loop to build popup menu controls for spectrum parameters
		PMpath=DecompSettingsFolder+":PM"+num2istr(i)
		SVAR/Z gPM=$PMpath
		SVAR/Z gPMvalue=$(PMpath+"list")
		if(SVAR_exists(gPM))
			TFPs.PM.popStr[i]=gPM
			TFPs.PM.popNum[i]=whichlistitem(gPM,gPMvalue)+1
		else
			TFPs.PM.popStr[i]=""
			TFPs.PM.popNum[i]=nan
		endif
	endfor

	TFPs.Xmin=TFgr.Xmin;TFPs.Xmax=TFgr.Xmax
	TFPs.Fmin=1/TFgr.PeriodMax;TFPs.Fmax=1/TFgr.PeriodMin
	TFPs.freqRes=1/TFgr.freqRes/(TFgr.Xmax-TFgr.Xmin)
	if(waveexists(TFgr.xw))
		wave TFPs.dw=$getwavesdatafolder(TFgr.dw,2);wave TFPs.xw=$getwavesdatafolder(TFgr.xw,2)
	endif
	wave TFPs.dw_int=$getwavesdatafolder(TFgr.dw_int,2)

//make result waves
	NewDataFolderPath(TFgr.TFDecompFolder)
	make/o $(TFgr.TFDecompFolder+":magnitude"),$(TFgr.TFDecompFolder+":BGnorm"),$(TFgr.TFDecompFolder+":COI")
	make/o $(TFgr.TFDecompFolder+":Frequency"),$(TFgr.TFDecompFolder+":Period"),$(TFgr.TFDecompFolder+":scale")
	TF_MakeRefs(TFgr,TFgr.win)
	wave TFPs.magnitude=$(TFgr.TFDecompFolder+":magnitude"),TFPs.coi=$(TFgr.TFDecompFolder+":COI")
	wave TFPs.frequency=$(TFgr.TFDecompFolder+":Frequency"),TFPs.period=$(TFgr.TFDecompFolder+":Period")
	wave TFPs.scale=$(TFgr.TFDecompFolder+":scale")
	
	DecompFunc(TFPs)

	TF_Sum(TFgr)

//background amplitude normalization
	duplicate/o TFgr.TFMag,TFgr.TFBGnorm
	TFgr.TFBGnorm=TFgr.TFMag[p][q]/TFgr.AR1Mag(y)
end

Function TF_SpectrumOptions(TFgr)	//adds controls specfic to spectral estimate methods
	struct TF_GlobalRefs &TFgr
	
	struct SpectrumParameters SPs
	SPs.getParams=1		//makes spectrum function return information needed to build parameter controls
	FuncRef SPECTRUM_prototype SpectrumFunc=$("SPECTRUM_"+TFgr.SpectrumType)
	SpectrumFunc(SPs)
	string SpecSettingsFolder=TFgr.SettingsFolder+":Spectrum:"+TFgr.SpectrumType
	NewDataFolderPath(SpecSettingsFolder)
	variable i
	string SVpath,PMpath,ctrlName

	for(i=0;i<3;i+=1)					//loop to build setvariable controls for spectrum parameters
		SVpath=SpecSettingsFolder+":SV"+num2istr(i)
		NVAR/Z gSV=$SVpath
		ctrlName="SpectrumSV"+num2istr(i+1)
		if(strlen(SPs.SV.name[i]))
			if(NVAR_exists(gSV))
				SPs.SV.value[i]=gSV
			else
				variable/g $SVpath=SPs.SV.value[i]
			endif
			SetVariable $ctrlName pos={255,25+i*25},size={150,20},bodywidth=70,proc=TFSetVariable,win=$TFgr.ctrlWin
			setvariable $ctrlName limits={SPs.SV.low[i],SPs.SV.high[i],SPs.SV.inc[i]},title=SPs.SV.name[i],value=$SVpath,win=$TFgr.ctrlWin
		else
			killvariables/z gSV
			killcontrol/W=$TFgr.ctrlWin $ctrlName
		endif
	endfor
	for(i=0;i<3;i+=1)				//loop to build popup menu controls for spectrum parameters
		PMpath=SpecSettingsFolder+":PM"+num2istr(i)
		SVAR/Z gPM=$PMpath
		string/g $(PMpath+"list")
		SVAR gPMvalue=$(PMpath+"list")
		ctrlName="SpectrumPM"+num2istr(i+1)
		if(strlen(SPs.PM.name[i]))
			if(SVAR_exists(gPM))
				SPs.PM.popStr[i]=gPM
			else
				string/g $PMpath=SPs.PM.popStr[i]
			endif
			gPMvalue=SPs.PM.value[i]
			PopupMenu $ctrlName,pos={505,25+i*25},proc=TFPopup,title=SPs.PM.name[i],win=$TFgr.ctrlWin
			PopupMenu $ctrlName value=#(PMpath+"list"),popvalue=SPs.PM.popStr[i],mode=1,win=$TFgr.ctrlWin
		else
			killstrings/z gPM,gPMvalue
			killcontrol/W=$TFgr.ctrlWin $ctrlName
		endif
	endfor
end

Function TF_InitiateSpectrum(TFgr)		//calculates spectrum
	struct TF_GlobalRefs &TFgr
	
	TF_AR1(TFgr)					//calculates expected AR1 red noise spectrum

	struct SpectrumParameters SPs
	SPs.getParams=0					//setting to calculate spectrum
	FuncRef SPECTRUM_prototype SpectrumFunc=$("SPECTRUM_"+TFgr.SpectrumType)
	string SpecSettingsFolder=TFgr.SettingsFolder+":Spectrum:"+TFgr.SpectrumType
	NewDataFolderPath(SpecSettingsFolder)
	variable i
	string SVpath,PMpath,ctrlName

	for(i=0;i<3;i+=1)		//loop to get values of setvariable parameters
		SVpath=SpecSettingsFolder+":SV"+num2istr(i)
		NVAR/Z gSV=$SVpath
		if(NVAR_exists(gSV))
			SPs.SV.value[i]=gSV
		else
			SPs.SV.value[i]=nan
		endif
	endfor
	for(i=0;i<3;i+=1)		//loop to get values of popup menu parameters
		PMpath=SpecSettingsFolder+":PM"+num2istr(i)
		SVAR/Z gPM=$PMpath
		SVAR/Z gPMvalue=$(PMpath+"list")
		if(SVAR_exists(gPM))
			SPs.PM.popStr[i]=gPM
			SPs.PM.popNum[i]=whichlistitem(gPM,gPMvalue)+1
		else
			SPs.PM.popStr[i]=""
			SPs.PM.popNum[i]=nan
		endif
	endfor
	
	SPs.Xmin=TFgr.Xmin;SPs.Xmax=TFgr.Xmax
	SPs.Fmin=1/TFgr.PeriodMax;SPs.Fmax=1/TFgr.PeriodMin
	SPS.freqRes=1/TFgr.freqRes/(TFgr.Xmax-TFgr.Xmin)
	if(waveexists(TFgr.xw))
		wave SPs.xw=$getwavesdatafolder(TFgr.xw,2)
	endif
	wave SPs.dw=$getwavesdatafolder(TFgr.dw,2)
	wave SPs.dw_int=$getwavesdatafolder(TFgr.dw_int,2)
	NewDataFolderPath(TFgr.SpectrumFolder)											//Create spectrum subfolder, if it doesn't exist
	make/o $(TFgr.SpectrumFolder+":Amplitude")
	wave SPs.magnitude=$(TFgr.SpectrumFolder+":Amplitude")
	
	SpectrumFunc(SPs)		//calls spectrum calculation wrapper function
	
	duplicate/o SPs.magnitude,$(TFgr.SpectrumFolder+":Frequency"),$(TFgr.SpectrumFolder+":Period")
	TF_MakeRefs(TFgr,TFgr.win)
	TFgr.PgramFrequency=x;TFgr.PgramPeriod=1/x
end

Function TF_AR1(TFgr)			//calculates expected AR1 red noise spectrum
	struct TF_GlobalRefs &TFgr
	
	struct AR1_Params AR1p
	AR1_getParamsSimple(TFgr.dw_int,AR1p)
	
	variable N=numpnts(TFgr.dw_int)
	newdatafolderPath(TFgr.AR1folder)
	make/o/n=((0.5/TFgr.interpRes)/(1/(TFgr.XMax-TFgr.XMin))) $(TFgr.AR1folder+":Amplitude"),$(TFgr.AR1folder+":period"),$(TFgr.AR1folder+":frequency")
	wave TFgr.AR1Mag=$(TFgr.AR1folder+":Amplitude"),TFgr.AR1period=$(TFgr.AR1folder+":period"),TFgr.AR1frequency=$(TFgr.AR1folder+":frequency")
	setscale/p x 0,1/(TFgr.XMax-TFgr.XMin),TFgr.AR1Mag,TFgr.AR1period,TFgr.AR1frequency
	TFgr.AR1Mag=AR1p.tau/TFgr.interpRes*AR1p.alpha*sqrt(1/(1+(2*pi*x*AR1p.tau)^2)/N)*pi/2	//ref?
//	TFgr.AR1Mag=sqrt((1-AR1p.gamm^2)/(1+AR1p.gamm^2-2*AR1p.gamm*cos(2*pi*p/N))/N)/2
	TFgr.AR1Mag*=sqrt(statsInvChiCDF(TFgr.sigLevel/100,2))												//spectrum should be chi2 distributed
	TFgr.AR1frequency=x;TFgr.AR1period=1/x
end

Function TF_Sum(TFgr)					//calculates spectrum by taking mean of time-frequency decomposition
	struct TF_GlobalRefs &TFgr
	variable dup		//if >0, this creates a duplicate of the current plot rather than updating
	
	variable i,npts=dimsize(TFgr.TFMag,0),ncols=dimsize(TFgr.TFMag,1)
	string name=GetWavesDataFolder(TFgr.TFMag,1)+"GlobalSpec"
	make/o/n=(ncols) $(name)
	wave TFgr.TFGlobalSpec=$(name)
	setscale/p x DimOffset(TFgr.TFMag,1),DimDelta(TFgr.TFMag,1),TFgr.TFGlobalSpec
	TFgr.TFGlobalSpec=0
	for(i=0;i<ncols;i+=1)
		ImageStats/G={0,npts-1,i,i} TFgr.TFMag
		TFgr.TFGlobalSpec[i]=V_rms
	endfor
end

function TF_UpdatePlot(TFgr[,dup])
	struct TF_GlobalRefs &TFgr
	variable dup		//if >0, this creates a duplicate of the current plot rather than updating
		
	string plotN
	variable tickInterval
	
	if(dup)				//if duplicating rather than updating
		plotN=uniquename(nameofwave(TFgr.dw)+"_TFplot",6,0)
		display/W=(5,164,705,764)/N=$plotN
		Modifygraph/W=$plotN margin(left)=72,margin(bottom)=72,margin(right)=(72)
	else
		plotN=TFgr.win
//Remove old plots:
		ClearGraph(win=plotN)
		ColorScale/N=AmpScale/K/W=$plotN
	endif
		
	Modifygraph/W=$plotN swapXY=0
	if(waveexists(TFgr.dw))											//if data wave has been selected, then update the plot
		if(WaveExists(TFgr.xw))	//plot dw vs xw, if one has been selected
			appendToGraph/W=$plotN/c=(0,0,0)/l=Data/b=Position TFgr.dw vs TFgr.xw
		else
			appendToGraph/W=$plotN/c=(0,0,0)/l=Data/b=Position TFgr.dw
		endif
		if(WaveExists(TFgr.dw_int) && (numpnts(TFgr.dw_int)>0))	//plot interpolated data, if it exists
			AppendToGraph/W=$plotN/l=Data/b=Position TFgr.dw_int
		endif
		setaxis/W=$plotN/A=2 Data
		setaxis/W=$plotN Position TFgr.Xmin,TFgr.Xmax

		//plot spectrum
		if(waveexists(TFgr.PgramMag) && waveexists(TFgr.PlotXPgram))
			AppendToGraph/W=$plotN/c=(0,0,0)/B=Amplitude/L=Period/VERT TFgr.PgramMag vs TFgr.PlotXPgram
		endif
		if(waveexists(TFgr.PlotTFdecomp) && waveexists(TFgr.PlotXTFdecomp))
			AppendImage/W=$plotN/l=Period/b=Position TFgr.PlotTFdecomp vs {*,TFgr.PlotXTFdecomp}
			AppendToGraph/W=$plotN/C=(0,0,65535)/B=Amplitude/L=Period/VERT TFgr.TFGlobalSpec vs TFgr.PlotXTFdecomp
			TF_LogColorTable(TFgr)
			if(numtype(TFgr.CTabMin))
				wavestats/q/m=1 TFgr.PlotTFdecomp;TFgr.CTabMin=(v_min+v_avg)/10
			endif
			if(numtype(TFgr.CTabMax))
				wavestats/q/m=1 TFgr.PlotTFdecomp;TFgr.CTabMax=(v_max)
			endif
			ModifyImage/W=$plotN $nameofwave(TFgr.PlotTFdecomp) ctab= {TFgr.CTabMin,TFgr.CTabMax,$TFgr.CTabMode,0}
			ModifyImage/W=$plotN $nameofwave(TFgr.PlotTFdecomp) lookup=TFgr.CTabMod
			ColorScale/W=$plotN/C/N=AmpScale/F=0/A=LB/E=1/X=0/Y=0 vert=0,widthPct=40,height=10,log=0,minor=1,fsize=10
			ColorScale/W=$plotN/C/N=AmpScale image=$nameofwave(TFgr.PlotTFdecomp)
		endif
		if(waveexists(TFgr.AR1Mag))			//if AR1 expected red noise spectrum has been calculated
			AppendToGraph/W=$plotN/C=(65535,0,0)/B=Amplitude/L=Period/VERT TFgr.AR1Mag vs TFgr.PlotXAR1
		endif

		Modifygraph/W=$plotN/Z axisenab(Data)={0.8,1},axisenab(Position)={0,0.8}
		if(!waveexists(TFgr.vertP))
			make/o/t $(TFgr.TFfolder+":vertP"),$(TFgr.TFfolder+":vertF")
			make/o $(TFgr.TFfolder+":vertPy"),$(TFgr.TFfolder+":vertFy")
			wave/t TFgr.vertP=$(TFgr.TFfolder+":vertP"),TFgr.vertF=$(TFgr.TFfolder+":vertF")
			wave TFgr.vertPy=$(TFgr.TFfolder+":vertPy"),TFgr.vertFy=$(TFgr.TFfolder+":vertFy")
		endif
		NewFreeAxis/W=$plotN/o/r Frequency
		if(whichListItem("Period",AxisList(TFgr.win))>0)
			strswitch(TFgr.VertAxis)						//differentiate between linear and log axes
				case "Linear frequency":
					setaxis/W=$plotN/R Period 1/TFgr.PeriodMin,1/TFgr.PeriodMax
					setaxis/W=$plotN/R Frequency 1/TFgr.PeriodMin,1/TFgr.PeriodMax
					ModifyGraph/W=$plotN log(Period)=0,log(Frequency)=0
					tickInterval=SigDigits((1/TFgr.PeriodMin-1/TFgr.PeriodMax)/(TFgr.NTicks),1)
					redimension/n=((1/TFgr.PeriodMin-1/TFgr.PeriodMax)/tickInterval) TFgr.vertF,TFgr.vertFy,TFgr.vertP,TFgr.vertPy
					TFgr.VertFy=(p+ceil((1/TFgr.PeriodMax)/TickInterval))*tickInterval
					TFgr.VertF=num2str(TFgr.VertFy)
					TFgr.VertPy=1/SigDigits(1/TFgr.vertFy,1);TFgr.VertP=num2str(1/TFgr.VertPy)
					ModifyGraph/W=$plotN userticks(Period)={TFgr.vertPy,TFgr.vertP}
//					ModifyGraph/W=$plotN userticks(Frequency)={TFgr.vertFy,TFgr.vertF}
//					TFgr.vertP[]=selectstring(mod((p),TFgr.LabelSpace),TFgr.vertP[p],"")
//					TFgr.vertF[]=selectstring(mod((p),TFgr.LabelSpace),TFgr.vertF[p],"")
					break
				case "Linear Period":
					setaxis/W=$plotN Period TFgr.PeriodMin,TFgr.PeriodMax
					setaxis/W=$plotN Frequency TFgr.PeriodMin,TFgr.PeriodMax
					ModifyGraph/W=$plotN log(Period)=0,log(Frequency)=0
					tickInterval=SigDigits((TFgr.PeriodMax-TFgr.PeriodMin)/(TFgr.NTicks),1)
					redimension/n=((TFgr.PeriodMax-TFgr.PeriodMin)/tickInterval) TFgr.vertP,TFgr.vertPy,TFgr.vertF,TFgr.vertFy
					TFgr.VertPy=(p+ceil(TFgr.PeriodMin/TickInterval))*TickInterval
					TFgr.VertP=num2str(TFgr.VertPy)
					TFgr.VertFy=1/SigDigits(1/TFgr.VertPy,1);TFgr.VertF=num2str(1/Tfgr.VertFy)
//					ModifyGraph/W=$plotN userticks(Period)={TFgr.vertPy,TFgr.vertP}
					ModifyGraph/W=$plotN userticks(Frequency)={TFgr.vertFy,TFgr.vertF}
//					TFgr.vertP[]=selectstring(mod((p),TFgr.LabelSpace),TFgr.vertP[p],"")
//					TFgr.vertF[]=selectstring(mod((p),TFgr.LabelSpace),TFgr.vertF[p],"")
					break
				case "Log(10)":
					setaxis/W=$plotN Period TFgr.PeriodMin,TFgr.PeriodMax
					setaxis/W=$plotN Frequency 1/TFgr.PeriodMin,1/TFgr.PeriodMax
					ModifyGraph/W=$plotN log(Period)=1,log(Frequency)=1
					break
				case "Log(2)":
					setaxis/W=$plotN Period TFgr.PeriodMin,TFgr.PeriodMax
					setaxis/W=$plotN Frequency 1/TFgr.PeriodMin,1/TFgr.PeriodMax
					ModifyGraph/W=$plotN log(Period)=2,log(Frequency)=2
					break
			endSwitch
			Modifygraph/W=$plotN/Z axisenab(Period)={0,0.75},axisenab(Frequency)={0,0.75}
		endif
		if(whichListItem("Amplitude",AxisList(TFgr.win))>0)
			ModifyGraph/W=$plotN/Z log(Amplitude)=whichlistitem(TFgr.AmpAxis,KPopupvalsAmpAxis)
			Modifygraph/W=$plotN axisenab(Amplitude)={0.85,1}
			wavestats/q/m=(1)/z/R=(1/TFgr.PeriodMax,1/TFgr.PeriodMin) TFgr.PgramMag
			setaxis/W=$plotN/a=2 Amplitude
		endif
		ModifyGraph/W=$plotN lblPos=60
		Modifygraph/W=$plotN freepos=0,minor=1
		ModifyGraph/W=$plotN  gfSize=12
		Modifygraph/W=$plotN nticks=TFgr.NTicks
		
		string ordUnits,datUnits=Waveunits(TFgr.dw,-1)
		if(waveexists(TFgr.xw))
			ordUnits=Waveunits(TFgr.xw,-1)
		else
			ordUnits=Waveunits(TFgr.dw,0)
		endif
		Label/Z/W=$plotN  Position,"ordinate ("+ordUnits+")"
		Label/Z/W=$plotN  Period,"wavelength ("+ordUnits+"/cycle)"
		Label/Z/W=$plotN  Frequency,"frequency (cycles/"+OrdUnits+")"
		ModifyGraph/W=$plotN lblRot(Frequency)=180
		ModifyGraph TickUnit=1
		Label/Z/W=$plotN  Amplitude,"magnitude ("+DatUnits+")"
		Label/Z/W=$plotN  Data,DatUnits
		if(cmpstr(TFgr.SwapXY,"vertical data plot")==0)
			Modifygraph/W=$plotN swapXY=1
			Modifygraph/W=$plotN lblRot(Frequency)=0
			ReverseAxisRange("Period",plotN)
		endif
	endif
end

function ReverseAxisRange(axisName,plotName)
	string axisName,plotName
	
	getaxis/W=$plotName/Q $axisName
	setaxis/Z/W=$plotName $axisName,v_max,v_min
end

function SigDigits(val,dig)			//utility that returns a value truncated to dig number of digits
	variable val,dig
	
	variable i=1,ret
	do
		ret=trunc(val*10^i)
		if(ret==0)
			i+=1
		elseif(ret>=10)
			i-=1
		elseif(numtype(ret))
			return nan
		else
			i=10^i*10^(dig-1)
			return round(val*i)/i
		endif
	while(1)
end

static Function NewDataFolderPath(path[,set])
	string path
	variable set
	
	variable depth=itemsinlist(path,":"),i
	string partial=stringfromlist(0,path,":")
	if(strlen(partial)==0)	//path is relative, beginning with a :
		partial="";i=1
	elseif(cmpstr("root",partial)==0) //path is full from root
		partial="root";i=1
	else						//path is relative, with no initial :
		partial="";i=0
	endif
	for(i=i;i<depth;i+=1)
		partial+=":"+possiblyquotename(cleanupname(StringFromList(i,path,":"),1))
		newdatafolder/o $partial
	endfor
	if(set)
		SetDataFolder $partial
	endif
end

static Function ClearGraph([win])
	string win
	
	if(paramisdefault(win))
		win=winname(0,1)
	endif
	dowindow $win
	if(v_flag==0)
		win=winname(0,1)
	endif
	
	string PlotList
	variable i,nwaves
	PlotList=TraceNameList(win,";",1);nwaves=ItemsInList(PlotList)
	for(i=nwaves-1;i>=0;i-=1)
		RemoveFromGraph/W=$win/Z $stringfromlist(i,PlotList)
	endfor
	PlotList=ImageNameList(win,";");nwaves=ItemsInList(PlotList)
	for(i=nwaves-1;i>=0;i-=1)
		RemoveImage/W=$win/Z $stringfromlist(i,PlotList)
	endfor
	PlotList=AxisList(win);nwaves=itemsinList(PlotList)
	for(i=nwaves-1;i>=0;i-=1)
		KillFreeAxis/W=$win $stringfromlist(i,PlotList)
	endfor
end

static Function RemoveFliers(rg,sd,lw[,xw])
	wave lw,xw;variable rg,sd
	
	variable np,i,nr=0,no=0,pmin,pmax
	
	np=numpnts(lw)
	duplicate/o lw,tempderiv
	rg/=2
	if(waveexists(xw))
		differentiate tempderiv /X=xw
	else
		differentiate tempderiv
		rg/=deltax(lw)
	endif
//	do
		for(i=0,nr=no;i<np;i+=1)
			if(waveexists(xw))
				pmin=binarysearch(xw,xw[i]-rg)+1;pmax=binarysearch(xw,xw[i]+rg)
				if(pmax==-2)
					pmax=np-1
				endif
				wavestats/q/r=[pmin,pmax] tempderiv
			else
				wavestats/q/r=[max(i-rg,0),min(i+rg,np-1)] tempderiv
			endif
			if((abs(tempderiv[i])>(sd*V_sdev))*(numtype(tempderiv[i])<2))
				lw[i-1,i+1]=NaN;tempderiv[i]=NaN;no+=1
			endif
		endfor
//	while(nr!=no)
	killwaves/z tempderiv
	return nr
end

static Function RemoveOutliers(rg,sd,lw[,xw])
	wave lw,xw;variable rg,sd
	
	variable np,i,no=0,nr=0,pmin,pmax
	
	np=numpnts(lw)
	if(waveexists(xw))
	else
		rg/=deltax(lw)
	endif
//	do
		for(i=0,nr=no;i<np;i+=1)
			if(waveexists(xw))
				pmin=binarysearch(xw,xw[i]-rg)+1;pmax=binarysearch(xw,xw[i]+rg)
				if(pmax==-2)
					pmax=np-1
				endif
				wavestats/q/r=[pmin,pmax] lw
			else
				wavestats/q/r=[max(i-rg,0),min(i+rg,np-1)] lw
			endif
			if((lw[i]>(V_avg+sd*V_sdev)) + (lw[i]<(V_avg-sd*V_sdev)))
				lw[i]=NaN;no+=1
			endif
		endfor
//	while(nr!=no)
	return nr
end


static Function ReplaceNaN(w,at,rg)
	wave w
	variable at	//method of filling: 
				//		0 to fill with the average of the closest values
				//		1 to linearly interpolate between closest values
	variable rg	//number of points to use in defining closest values
	
	Variable i,j,avg,np=numpnts(w)
	
	i=0
	do
		wavestats/q/r=[i,min(i+99,np-1)] w
		if(V_numNaNs>0)
			i-=1
			do
				i+=1
			while((numtype(w[i])==0) & (i<np))
			j=0
			do
				j+=1
			while((numtype(w[i+j])!=0) & (i+j<np))
			if(i==0)
				w[i,i+j-1]=w[i+j]
			elseif(i+j==np)
				w[i,i+j-1]=w[i-1]
			elseif(at)
				wavestats/q/r=[max(i-rg,0),max(i-1,0)] w;avg=V_avg
				wavestats/q/r=[min(i+j,np-1),min(i+j+rg-1,np-1)] w
				w[i,i+j-1]=avg-((avg-V_avg)/j)*(p-i+1)
			else
				wavestats/q/r=[max(i-rg,0),min(i+j+rg-1,np-1)] w
				w[i,i+j-1]=v_avg
			endif
			i+=j
		else
			i+=100
		endif
	while(i<np)
end

